{ $Header: /MidiComp/MIDITYPE.PAS 2     10/06/97 7:33 Davec $ }

{ Written by David Churcher <dchurcher@cix.compulink.co.uk>, released to the public domain. }

// 2020-04-13: Changed by Blue Hell for use in Delphi 10
//             changes released to the public domain

unit Miditype;

interface

uses

  System.Classes, Winapi.Windows, Winapi.Messages, Winapi.MMSystem,

  MidiDefs, Circbuf;


type


  // MIDI input/output event
  TMyMidiEvent = record
  public
    MidiMessage  : Byte;            // MIDI message status byte
    Data1        : Byte;            // MIDI message data 1 byte
    Data2        : Byte;            // MIDI message data 2 byte
    Time         : DWORD;           // Time in ms since midiInOpen .. or is it ... 'some' time it is anyway
    SysexLength  : Word;            // Length of sysex data (0 if none)
    Sysex        : PByte;           // Pointer to sysex data buffer
  public
    procedure   Destroy;
    procedure   Clear;
  end;

  PMyMidiEvent = ^TMyMidiEvent;


  // Encapsulates the MIDIHDR with its memory handle and sysex buffer

  PMyMidiHdr = ^TMyMidiHdr;

  TMyMidiHdr = class( TObject)
  public
    hdrHandle     : THandle;
    hdrPointer    : PMIDIHDR;
    sysexHandle   : THandle;
    sysexPointer  : Pointer;
  public
    constructor Create(BufferSize: Word);
    destructor  Destroy;                                                                                       override;
  end;


implementation


    procedure   TMyMidiEvent.Destroy;
    // Free any sysex buffer associated with the event
    begin
      if   Sysex <> nil
      then Freemem( Sysex, SysexLength);
    end;


    procedure   TMyMidiEvent.Clear;
    begin
      MidiMessage  := 0;
      Data1        := 0;
      Data2        := 0;
      Time         := 0;
      SysexLength  := 0;
      Sysex        := nil;
    end;


    constructor TMyMidiHdr.Create( BufferSize: Word);
    // Allocate memory for the sysex header and buffer
    begin
      inherited Create;

      if   BufferSize > 0
      then begin
        hdrPointer   := GlobalSharedLockedAlloc( sizeof( TMIDIHDR), hdrHandle  );
        sysexPointer := GlobalSharedLockedAlloc( BufferSize       , sysexHandle);
        hdrPointer^.lpData         := sysexPointer;
        hdrPointer^.dwBufferLength := BufferSize;
      end;
    end;


    destructor TMyMidiHdr.Destroy;
    begin
      GlobalSharedLockedFree( sysexHandle, sysexPointer );
      GlobalSharedLockedFree( hdrHandle  , hdrPointer   );
      inherited;
    end;

end.

